@@ -1,23 +1,23 @@
-/***************************************************************************
-                           playlist.cpp
-                           -------------------
-    begin                : Die Apr 22 2003
-    revision             : $Revision: 1.88 $
-    last modified        : $Date: 2005/08/22 19:36:46 $ by $Author: juergenk $
-    copyright            : (C) 2003-2005 by Jürgen Kofler
-    email                : kaffeine@gmx.net
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
-
-
+/*
+ * playlist.cpp
+ *
+ * Copyright (C) 2003-2005 Jürgen Kofler <kaffeine@gmx.net>
+ * Copyright (C) 2005-2006 Christophe Thommeret <hftom@free.fr>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+ */
 
 #include <krandomsequence.h>
 #include <kapplication.h>
@@ -42,6 +42,8 @@
 #include <dcopref.h>
 #include <ktoolbar.h>
 #include <kcolordialog.h>
+#include <kurlcombobox.h>
+#include <kurlcompletion.h>
 
 #include <qlabel.h>
 #include <qclipboard.h>
@@ -121,7 +123,7 @@ void RollTitle::setTitle( QString t )
 	p.fillRect( 0, 0, titlePix.width(), titlePix.height(), back );
 	p.setPen( fore );
 	p.setFont( font() );
-	p.drawText( QRect(0,(titlePix.height()-QFontInfo(font()).pixelSize())/2,titlePix.width(),QFontInfo(font()).pixelSize()), Qt::AlignAuto|Qt::SingleLine, t );
+	p.drawText( QRect(0,(titlePix.height()-QFontInfo(font()).pointSize())/2,titlePix.width(),QFontInfo(font()).pointSize()*2), Qt::AlignAuto|Qt::SingleLine, t );
 	p.end();
 }
 
@@ -185,9 +187,9 @@ struct CoverPopup : public QWidget
 CoverFrame::CoverFrame( QWidget *parent ) : QFrame( parent )
 {
 	imagePath = "";
-	QString path = locate("appdata", "nocover");
+	QString path = locate("appdata", "nocover.png");
 	if ( !path )
-		path = locate("data", "kaffeine/nocover");
+		path = locate("data", "kaffeine/nocover.png");
 	noCoverPix.convertFromImage( QImage(path) );
 	setPaletteBackgroundPixmap( noCoverPix );
 }
@@ -218,7 +220,6 @@ void CoverFrame::popup( const QPixmap &pix ) const
 {
     QPoint mouse  = QCursor::pos();
     QRect desktop = KApplication::desktop()->screenGeometry(mouse);
-
     int x = mouse.x();
     int y = mouse.y();
     int height = pix.height() + 4;
@@ -288,7 +289,6 @@ PlayList::PlayList( QWidget* parent, QObject *objParent, const char *name ) : Ka
 	panel = new QVBox( hSplit );
 	playlist = new QWidget( hSplit );
 	hSplit->moveToFirst( panel );
-	hSplit->moveToLast( playlist );
 	hSplit->setResizeMode( panel, QSplitter::KeepSize );
 
 	vSplit = new QSplitter( QSplitter::Vertical, panel );
@@ -298,6 +298,21 @@ PlayList::PlayList( QWidget* parent, QObject *objParent, const char *name ) : Ka
 	QVBoxLayout *v = new QVBoxLayout( vb, 0, 0 );
 	v->addWidget( tb );
 	tb->setIconSize( 16 );
+
+	browserComb = new KURLComboBox( KURLComboBox::Directories, true, vb );
+	browserComb->setCompletionObject( new KURLCompletion( KURLCompletion::DirCompletion ) );
+	browserComb->setAutoDeleteCompletionObject( true );
+        browserComb->setMaxItems( 10 );
+        browserComb->setFocusPolicy(QWidget::ClickFocus);
+
+
+       /* if (!m_medium)
+            m_combo->lineEdit()->setText( location->path() );
+        else
+            m_combo->lineEdit()->setText( "/" );*/
+
+	QVBoxLayout *v1 = new QVBoxLayout( vb, 3, 3 );
+	v1->addWidget( browserComb );
 	fileBrowser = new KDirOperator( KURL("/"), vb );
 	fileBrowser->setupMenu( KDirOperator::SortActions|KDirOperator::NavActions|KDirOperator::ViewActions );
 	view = new KFileIconView( fileBrowser, "view" );
@@ -306,7 +321,6 @@ PlayList::PlayList( QWidget* parent, QObject *objParent, const char *name ) : Ka
 	fileBrowser->setView( view );
 	fileBrowser->setViewConfig( KGlobal::config(), "PlaylistFileBrowser" );
 	fileBrowser->setOnlyDoubleClickSelectsFiles( true );
-	QVBoxLayout *v1 = new QVBoxLayout( vb, 3, 3 );
 	v1->addWidget( fileBrowser );
 	v->addLayout( v1 );
 
@@ -384,7 +398,7 @@ PlayList::PlayList( QWidget* parent, QObject *objParent, const char *name ) : Ka
 	QLabel* playlistLabel = new QLabel(i18n("Playlist:"), playlist);
 	h2->addWidget(playlistLabel);
 	m_playlistSelector = new KComboBox( playlist );
-	m_playlistSelector->setMinimumWidth(160);
+	m_playlistSelector->setMinimumWidth(10);
 	m_playlistSelector->setSizePolicy( QSizePolicy (QSizePolicy::MinimumExpanding, QSizePolicy::Preferred) );
 	m_playlistSelector->setEditable(true);
 	m_playlistSelector->setDuplicatesEnabled(false);
@@ -418,7 +432,10 @@ PlayList::PlayList( QWidget* parent, QObject *objParent, const char *name ) : Ka
 
 	connect(coverFrame, SIGNAL(changeCurrentCover()), this, SLOT(chooseCurrentCover()));
 	connect(coverFrame, SIGNAL(gallery()), this, SLOT(showGallery()));
-	connect(fileBrowser, SIGNAL(fileSelected(const KFileItem*)), this, SLOT(fileSelected(const KFileItem*)) );
+	connect(browserComb, SIGNAL(urlActivated( const KURL& )), this, SLOT(setBrowserURL( const KURL& )));
+	connect(browserComb, SIGNAL(returnPressed( const QString& )), this, SLOT(setBrowserURL( const QString& )));
+	connect(fileBrowser, SIGNAL(urlEntered( const KURL& )), this, SLOT(browserURLChanged( const KURL& )));
+	connect(fileBrowser, SIGNAL(fileSelected(const KFileItem*)), this, SLOT(fileSelected(const KFileItem*)));
 	connect(m_playlistFilter, SIGNAL(textChanged(const QString&)), this, SLOT(slotFindText(const QString&)));
 	connect(m_playlistSelector, SIGNAL(returnPressed(const QString&)), this, SLOT(slotNewPlaylistName(const QString&)));
 	connect(m_playlistSelector, SIGNAL(activated(int)),this, SLOT(slotPlaylistActivated(int)));
@@ -436,6 +453,14 @@ PlayList::PlayList( QWidget* parent, QObject *objParent, const char *name ) : Ka
 	connect(m_list->header(), SIGNAL(clicked(int)), this, SLOT(slotSort(int)));
 }
 
+void PlayList::togglePanel()
+{
+	if ( panel->isHidden() )
+		panel->show();
+	else
+		panel->hide();
+}
+
 void PlayList::getTargets( QStringList &uiNames, QStringList &iconNames, QStringList &targetNames )
 {
 	uiNames.append( i18n("Play Playlist") );
@@ -461,42 +486,31 @@ void PlayList::setFileFilter( const QString& filter )
 
 PlayList::~PlayList()
 {
-	saveConfig( KGlobal::config() );
 	delete m_list;
 }
 
-void PlayList::toggleLayout( bool b )
-{
-	QValueList<int> sizes = hSplit->sizes();
-	int i = sizes[0];
-	sizes[0] = sizes[1];
-	sizes[1] = i;
-
-	if ( b ) {
-		hSplit->moveToLast( panel );
-		hSplit->moveToFirst( playlist );
-		hSplit->setSizes( sizes );
-	}
-	else {
-		hSplit->moveToLast( playlist );
-		hSplit->moveToFirst( panel );
-		hSplit->setSizes( sizes );
-	}
-	alternateLayout = b;
-}
-
 void PlayList::setupActions()
 {
 	m_repeat = new KToggleAction(i18n("&Repeat"), 0 , CTRL|Key_E, this, SLOT(slotRepeat()), actionCollection(),"playlist_repeat");
 	m_repeat->setStatusText(i18n("Loop playlist"));
 	m_shuffle = new KToggleAction(i18n("Sh&uffle"), 0 , CTRL|Key_R, this, SLOT(slotShuffle()), actionCollection(),"playlist_shuffle");
 	m_shuffle->setStatusText(i18n("Play items in random order"));
+	m_autoCover = new KToggleAction(i18n("Autodownload covers"), 0 , 0, this, SLOT(slotAutoCover()), actionCollection(),"playlist_autocover");
+	m_autoCover->setStatusText(i18n("Automatic dowloading of covers"));
+	m_showPlayer = new KToggleAction(i18n("Don't switch to player window"), 0 , 0, this, SLOT(slotShowPlayer()), actionCollection(),"playlist_showplayer");
+	m_showPlayer->setStatusText(i18n("Don't switch automatically to player window"));
+	new KAction(i18n("&Clear Current Playlist"), "eraser", 0, this, SLOT(slotClearList()), actionCollection(), "playlist_clear");
 	new KAction(i18n("Ne&w Playlist"), "filenew", 0, this, SLOT(slotNewList()), actionCollection(), "playlist_new");
 	new KAction(i18n("&Import Playlist..."), "project_open", 0, this, SLOT(slotPlaylistLoad()), actionCollection(), "playlist_load");
 	new KAction(i18n("&Save Current Playlist As..."), "filesaveas", 0, this, SLOT(slotPlaylistSaveAs()), actionCollection(), "playlist_save_as");
 	new KAction(i18n("Re&move Current Playlist"), "fileclose", 0, this, SLOT(slotPlaylistRemove()), actionCollection(), "playlist_remove");
 }
 
+void PlayList::slotClearList()
+{
+	clearList();
+}
+
 void PlayList::slotRepeat()
 {
 	setEndless(m_repeat->isChecked());
@@ -507,6 +521,15 @@ void PlayList::slotShuffle()
 	setRandom(m_shuffle->isChecked());
 }
 
+void PlayList::slotAutoCover()
+{
+	m_cover = m_autoCover->isChecked();
+}
+
+void PlayList::slotShowPlayer()
+{
+}
+
 void PlayList::slotToggleShuffle()
 {
 	m_shuffle->setChecked(!m_shuffle->isChecked());
@@ -573,12 +596,12 @@ void PlayList::loadConfig(KConfig* config)
 	b = config->readBoolEntry("Random Mode", false);
 	m_shuffle->setChecked(b);
 	setRandom(b);
+	b = config->readBoolEntry("AutoCover", false);
+	m_autoCover->setChecked(b);
+	m_cover = b;
+	b = config->readBoolEntry("DontShowPlayer", false);
+	m_showPlayer->setChecked(b);
 
-	alternateLayout = config->readBoolEntry( "AlternateLayout", false );
-	if ( alternateLayout ) {
-		hSplit->moveToLast( panel );
-		hSplit->moveToFirst( playlist );
-	}
 	sl = config->readIntListEntry("HSplitSizes");
 	if ( sl.count() )
 		hSplit->setSizes( sl );
@@ -612,6 +635,12 @@ void PlayList::loadConfig(KConfig* config)
 	}
 	fileBrowser->readConfig( config, "PlaylistFileBrowser" );
 	fileBrowser->setURL( KURL(config->readEntry( "FileBrowserURL", "/" )), true );
+	browserComb->setURL( KURL(config->readEntry( "FileBrowserURL", "/" )) );
+}
+
+void PlayList::saveConfig()
+{
+	saveConfig( KGlobal::config() );
 }
 
 void PlayList::saveConfig(KConfig* config)
@@ -620,7 +649,8 @@ void PlayList::saveConfig(KConfig* config)
 	config->setGroup("Playlist");
 	config->writeEntry("Endless Mode", m_repeat->isChecked());
 	config->writeEntry("Random Mode", m_shuffle->isChecked());
-	config->writeEntry("AlternateLayout", alternateLayout );
+	config->writeEntry("AutoCover", m_autoCover->isChecked());
+	config->writeEntry("DontShowPlayer", m_showPlayer->isChecked());
 	config->writeEntry( "HSplitSizes", hSplit->sizes() );
 	config->writeEntry( "VSplitSizes", vSplit->sizes() );
 	QStringList list;
@@ -942,6 +972,31 @@ void PlayList::fileSelected( const KFileItem *item )
 	}
 }
 
+void PlayList::setBrowserURL( const QString &path )
+{
+	fileBrowser->setURL( KURL(path), true );
+}
+
+void PlayList::setBrowserURL( const KURL &path )
+{
+	fileBrowser->setURL( path, true );
+}
+
+void PlayList::browserURLChanged( const KURL &u )
+{
+	QString url;
+
+	 if ( u.isLocalFile() )
+	 	url = u.path();
+	 else
+	 	url = u.prettyURL();
+
+	QStringList urls = browserComb->urls();
+	urls.remove( url ); // do not duplicate
+	urls.prepend( url );
+	browserComb->setURLs( urls, KURLComboBox::RemoveBottom );
+}
+
 void PlayList::add(const QString& url, QListViewItem* after)
 {
 	add(QStringList(url), after);
@@ -959,7 +1014,7 @@ void PlayList::add(const QStringList& urlList, QListViewItem* after)
 	KURL tmpKURL;
 	bool playlist;
 	MRL mrl;
-	MRL::List mrlList;
+	QValueList<MRL> mrlList;
 	QString mediadevice="";
 
 	KProgressDialog* progress = NULL;
@@ -1027,6 +1082,7 @@ void PlayList::add(const QStringList& urlList, QListViewItem* after)
 			if (!QFile::exists(mrl.kurl().path()))
 				continue;
 			mrl.setTitle(mrl.kurl().fileName());
+			mrl.setURL(mrl.kurl().path());
 		}
 		else
 		{
@@ -1097,8 +1153,8 @@ void PlayList::add(const QStringList& urlList, QListViewItem* after)
 				}
 				if (playlist)
 				{
-					MRL::List::ConstIterator end(mrlList.end());
-					for (MRL::List::ConstIterator it = mrlList.begin(); it != end; ++it)
+					QValueList<MRL>::ConstIterator end(mrlList.end());
+					for (QValueList<MRL>::ConstIterator it = mrlList.begin(); it != end; ++it)
 						after = insertItem(after, *it);
 					continue;
 				}
@@ -1112,8 +1168,8 @@ void PlayList::add(const QStringList& urlList, QListViewItem* after)
 				kdDebug() << "PlayList: Try loading ram playlist\n";
 				if (PlaylistImport::ram(mrl, mrlList, mainWidget->parentWidget()))
 				{
-					MRL::List::ConstIterator end(mrlList.end());
-					for (MRL::List::ConstIterator it = mrlList.begin(); it != end; ++it)
+					QValueList<MRL>::ConstIterator end(mrlList.end());
+					for (QValueList<MRL>::ConstIterator it = mrlList.begin(); it != end; ++it)
 						after = insertItem(after, *it);
 					continue;
 				}
@@ -1221,16 +1277,17 @@ void PlayList::add(const QStringList& urlList, QListViewItem* after)
 						        endsWith(*it, ".asc", false) || endsWith(*it, ".sub", false) )
 							matches = true;
 
-						if(matches)
+						if(matches) {
 							subs.append(*it); //This is a subtitle for our movie
+						}
 						matches = false;
 					}
 				}
 
-				subs.prepend(i18n("(no subtitles)"));
-				subs.append(i18n("Other subtitle..."));
-				if((subs.count() >= 3))
-				{
+				if((subs.count() > 1)) {
+					subs.prepend(i18n("(no subtitles)"));
+					subs.append(i18n("Other subtitle..."));
+					subtitleURL = QString::null;
 					SubtitleChooser *sc = new SubtitleChooser(subs, filename, mainWidget);
 
 				THERE:     if(sc->exec() == QDialog::Accepted)
@@ -1253,15 +1310,18 @@ void PlayList::add(const QStringList& urlList, QListViewItem* after)
 						}
 					}
 					delete sc;
+					subs.remove(i18n("(no subtitles)"));
+					subs.remove(i18n("Other subtitle..."));
 				}
-				subs.remove(i18n("(no subtitles)"));
-				subs.remove(i18n("Other subtitle..."));
+
 				//Add the full path to the subtitle name
 				for (unsigned int i = 0; i < subs.size(); i++ )
 					if(!subs[i].startsWith(d->path()))
 						subs[i] = d->path()+"/"+subs[i];
 
 				mrl.setSubtitleFiles(subs);
+				if ( subs.count()==1 )
+					subtitleURL = subs[0]; // Use this one.
 				if (!subtitleURL.isNull())
 				{
 					kdDebug() << "PlayList: Use subtitle file: " << subtitleURL << " for: " << mrl.url() << endl;
@@ -1286,10 +1346,10 @@ void PlayList::add(const QStringList& urlList, QListViewItem* after)
 	updateStatus();
 }
 
-void PlayList::add(const MRL::List& mrlList, QListViewItem* after)
+void PlayList::add(const QValueList<MRL>& mrlList, QListViewItem* after)
 {
-	MRL::List::ConstIterator end(mrlList.end());
-	for (MRL::List::ConstIterator it = mrlList.begin(); it != end; ++it)
+	QValueList<MRL>::ConstIterator end(mrlList.end());
+	for (QValueList<MRL>::ConstIterator it = mrlList.begin(); it != end; ++it)
 		after = insertItem(after, *it);
 	updateStatus();
 	if (m_random) createRandomList();
@@ -1623,10 +1683,12 @@ void PlayList::setCover( QString s1, QString s2, bool forceFetch )
 			coverFrame->setCoverPixmap( "" );
 		if ( google )
 			return;
-		google = new GoogleFetcher( s1, s2 );
-		pix = google->pixmap( forceFetch );
-		delete google;
-		google = NULL;
+		if ( m_cover || forceFetch ) {
+			google = new GoogleFetcher( s1, s2 );
+			pix = google->pixmap( forceFetch );
+			delete google;
+			google = NULL;
+		}
 		if ( !QDir( s+"covers" ).exists() )
 			QDir().mkdir( s+"covers" );
 		if ( !pix.isNull() ) {
@@ -1636,10 +1698,10 @@ void PlayList::setCover( QString s1, QString s2, bool forceFetch )
 			if ( s1==mrl.artist().stripWhiteSpace() && s2==mrl.album().stripWhiteSpace() )
 				coverFrame->setCoverPixmap( fname );
 		}
-		else if ( !forceFetch ) {
-			QString path = locate("appdata", "nocover");
+		else if ( !forceFetch && m_cover ) {
+			QString path = locate("appdata", "nocover.png");
 			if ( !path )
-				path = locate("data", "kaffeine/nocover");
+				path = locate("data", "kaffeine/nocover.png");
 			KIO::file_copy( KURL::fromPathOrURL( path ), KURL::fromPathOrURL( fname ), -1, true );
 		}
 	}
@@ -1701,8 +1763,8 @@ void PlayList::updateStatus()
 	if (isQueueMode())
 	{
 		QTime total;
-		MRL::List::ConstIterator end(m_queue.end());
-		for (MRL::List::ConstIterator it = m_queue.begin(); it != end; ++it)
+		QValueList<MRL>::ConstIterator end(m_queue.end());
+		for (QValueList<MRL>::ConstIterator it = m_queue.begin(); it != end; ++it)
 			total = total.addSecs(QTime().secsTo((*it).length()));
 		status = i18n("Queue: %1 Entries, Playtime: %2").arg(m_queue.count()).arg(total.toString("h:mm:ss"));
 	}
@@ -1802,6 +1864,12 @@ void PlayList::slotNewPlaylistName(const QString& text)
 		mainWidget->parentWidget()->setFocus();
 }
 
+QString PlayList::queryCurrentPlaylist()
+{
+	QString pl = m_playlistSelector->text(m_currentPlaylist);
+	return (m_playlistDirectory + pl + ".kaffeine");
+}
+
 void PlayList::saveCurrentPlaylist()
 {
 	QString pl = m_playlistSelector->text(m_currentPlaylist);
@@ -2110,7 +2178,7 @@ void PlayList::slotPlaylistFromSelected()
 	QPtrList<QListViewItem> selected;
 	selected = m_list->selectedItems();
 
-	MRL::List mrlList;
+	QValueList<MRL> mrlList;
 
 	for (uint i=0; i<selected.count(); i++)
 	{
