@@ -1,19 +1,23 @@
-/***************************************************************************
-                           playlist.h
-                             -------------------
-    begin                : Die Apr 22 2003
-    copyright            : (C) 2003-2005 by Jrgen Kofler <kaffeine@gmx.net>
-    copyright            : (C) 2005-2006 by Christophe Thommeret <hftom@free.fr>
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/*
+ * playlist.h
+ *
+ * Copyright (C) 2003-2005 Jürgen Kofler <kaffeine@gmx.net>
+ * Copyright (C) 2005-2006 Christophe Thommeret <hftom@free.fr>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+ */
 
 #ifndef PLAYLIST_H
 #define PLAYLIST_H
@@ -136,8 +140,10 @@ public:
 	bool trackNumber( int, MRL& );
 	bool playbackFinished( MRL& );
 	void getTargets( QStringList &uiNames, QStringList &iconNames, QStringList &targetNames );
+	void togglePanel();
 	bool execTarget( const QString& );
 	void saveConfig();
+	bool showPlayer() {return !m_showPlayer->isChecked();}
 	//***************************************
 
 public:
@@ -153,7 +159,7 @@ public:
 	/* insert a KURL(list) after a specific item */
 	void add(const QString& url, QListViewItem* after);
 	void add(const QStringList& urls, QListViewItem* after);
-	void add(const MRL::List&, QListViewItem* after);
+	void add(const QValueList<MRL>&, QListViewItem* after);
 
 	bool isQueueMode() { return m_queue.count(); }
 
@@ -211,6 +217,7 @@ private slots:
 	void slotRepeat();
 	void slotShuffle();
 	void slotAutoCover();
+	void slotShowPlayer();
 	void slotPlaylistLoad();
 	void slotPlaylistSaveAs();
 	void slotPlaylistRemove();
@@ -255,9 +262,9 @@ private:
 	QString coverImageFormat;
 	RollTitle *roller;
 
-	KToggleAction *m_repeat, *m_shuffle, *m_autoCover;
+	KToggleAction *m_repeat, *m_shuffle, *m_autoCover, *m_showPlayer;
 
-	MRL::List m_queue;
+	QValueList<MRL> m_queue;
 
 	uint m_playTime;
 	uint m_playTimeVisible;
@@ -323,5 +330,4 @@ private:
 	QListBox *table;
 };
 
-
-#endif
+#endif /* PLAYLIST_H */
