@@ -167,6 +167,7 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 	channelsCb->setItemsRenameable( true );
 	channelsCb->addColumn( i18n("Number") );
 	channelsCb->addColumn( i18n("Name") );
+	channelsCb->addColumn( i18n("Source") );
 	channelsCb->setAllColumnsShowFocus( true );
 	channelsCb->setSizePolicy( QSizePolicy (QSizePolicy::Preferred, QSizePolicy::MinimumExpanding) );
 	//channelsCb->setEnabled( false );
@@ -260,6 +261,16 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 
 
 
+void DvbPanel::togglePanel()
+{
+	if ( panel->isHidden() )
+		panel->show();
+	else
+		panel->hide();
+}
+
+
+
 void DvbPanel::diskStatus()
 {
 	double freemb;
@@ -269,7 +280,7 @@ void DvbPanel::diskStatus()
 		fprintf(stderr,"Couldn't get file system statistics\n");
 		return;
 	}
-	
+
 	freemb = (double)(((double)(buf.f_bavail)*(double)(buf.f_bsize))/(1024.0*1024.0));
 	if ( freemb<300 )
 		emit showOSD( i18n("Warning: low disc space")+QString(" (%1").arg((int)freemb)+i18n("MB")+")", 5000, 3 );
@@ -525,10 +536,10 @@ void DvbPanel::dumpEvents()
 			events = dvb.at(i)->dvbEvents;
 			for ( m=0; m<(int)events->srcList.count(); m++ ) {
 				slist = events->srcList.at(m);
-				tt << slist->source+" : "+s.setNum( slist->tsidList.count() )+" TS with events.\n";
+				tt << slist->source+QString(" : %1 TS with events.\n").arg(slist->tsidList.count());
 				for ( n=0; n<(int)slist->tsidList.count(); n++ ) {
 					tlist = slist->tsidList.at(n);
-					tt << "TSID "+s.setNum( tlist->tsid )+" : "+s.setNum( tlist->events.count() )+" events\n";
+					tt << QString("TSID %1 : %2 events.\n").arg(tlist->tsid).arg(tlist->events.count());
 					k+= tlist->events.count();
 				}
 			}
@@ -547,10 +558,10 @@ void DvbPanel::dumpEvents()
 					for ( j=0; j<(int)tlist->events.count(); j++ ) {
 						desc = tlist->events.at(j);
 						tt << "Source: "+desc->source+"\n";
-						tt << "Network ID: "+s.setNum(desc->nid)+"\n";
-						tt << "Transport Stream ID: "+s.setNum(desc->tsid)+"\n";
-						tt << "Service ID: "+s.setNum(desc->sid)+"\n";
-						tt << "Event ID: "+s.setNum(desc->eid)+"\n";
+						tt << QString("Network ID: %1\n").arg(desc->nid);
+						tt << QString("Transport Stream ID: %1\n").arg(desc->tsid);
+						tt << QString("Service ID: %1\n").arg(desc->sid);
+						tt << QString("Event ID: %1\n").arg(desc->eid);
 						tt << "Title: "+desc->title+"\n";
 						tt << "Subtitle: "+desc->subtitle+"\n";
 						tt << desc->startDateTime.toString( "dd-MM-yyyy hh:mm:ss" )+"\n";
@@ -735,17 +746,22 @@ void DvbPanel::dvbOSD(ChannelDesc liveChannel, DvbStream *d, int timeShift /* =
 	int i=0, j, k;
 	int first=0;
 
-	if ( !d )
+	if ( !d ) {
+		//fprintf(stderr,"DVBOSD : d==null\n");
 		return;
+	}
 
-	if ( liveChannel.name.isEmpty() )
+	if ( liveChannel.name.isEmpty() ) {
+		//fprintf(stderr,"DVBOSD : channelName==empty\n");
 		return;
+	}
 
 	if ( osdMode==2 ) {
 		osdTimer.stop();
 		osdMode = 0;
 		list.append( "STOP" );
 		emit showDvbOSD( s.setNum( liveChannel.num )+" - "+liveChannel.name, list );
+		//fprintf(stderr,"DVBOSD : mode==2\n");
 		return;
 	}
 
@@ -781,13 +797,17 @@ void DvbPanel::dvbOSD(ChannelDesc liveChannel, DvbStream *d, int timeShift /* =
 	events->mutex.lock();
 	for ( j=0; j<(int)events->srcList.count(); j++ ) {
 		if ( events->srcList.at(j)->source==liveChannel.tp.source ) {
-			slist = events->srcList.at(i);
+			slist = events->srcList.at(j);
 			break;
 		}
 	}
 	if ( !slist ) {
+		//fprintf(stderr,"DVBOSD : slist==null\n");
 		goto end;
 	}
+	else {
+		//fprintf(stderr,"DVBOSD : liveChannel.tp.source==%s\n",liveChannel.tp.source.ascii());
+	}
 
 	for ( j=0; j<(int)slist->tsidList.count(); j++ ) {
 		tlist = slist->tsidList.at(j);
@@ -797,6 +817,7 @@ void DvbPanel::dvbOSD(ChannelDesc liveChannel, DvbStream *d, int timeShift /* =
 		}
 	}
 	if ( !currentEvents ) {
+		//fprintf(stderr,"DVBOSD : currentEvents==null\n");
 		goto end;
 	}
 
@@ -917,11 +938,7 @@ void DvbPanel::setConfig()
 	}
 
 	for ( i=0; i<(int)dvbConfig->devList.count(); i++ ) {
-		d = new DvbStream( dvbConfig->devList.at(i)->adapter, dvbConfig->devList.at(i)->tuner, dvbConfig->devList.at(i)->type, dvbConfig->devList.at(i)->source, dvbConfig->devList.at(i)->canAutoscan, dvbConfig->defaultCharset );
-		d->setLnb( 0, dvbConfig->devList.at(i)->lnb0 );
-		d->setLnb( 1, dvbConfig->devList.at(i)->lnb1 );
-		d->setLnb( 2, dvbConfig->devList.at(i)->lnb2 );
-		d->setLnb( 3, dvbConfig->devList.at(i)->lnb3 );
+		d = new DvbStream( dvbConfig->devList.at(i), dvbConfig->defaultCharset );
 		d->setPlug( plug );
 		dvb.append( d );
 		connect( d, SIGNAL(shifting(bool)), this, SLOT(setShiftLed(bool)) );
@@ -960,7 +977,7 @@ void DvbPanel::setConfig()
 
 void DvbPanel::showConfigDialog()
 {
-	int ret, i;
+	int ret;
 
 loop:
 	if ( !dvbConfig->haveData() ) {
@@ -979,13 +996,6 @@ loop:
 	disconnect( dlg.dumpBtn, SIGNAL(clicked()), this, SLOT(dumpEvents()) );
 	if ( ret==DvbConfigDialog::Rejected )
 		return;
-	for ( i=0; i<(int)dvbConfig->devList.count(); i++ ) {
-		dvb.at(i)->setSources( dvbConfig->devList.at(i)->source );
-		dvb.at(i)->setLnb( 0, dvbConfig->devList.at(i)->lnb0 );
-		dvb.at(i)->setLnb( 1, dvbConfig->devList.at(i)->lnb1 );
-		dvb.at(i)->setLnb( 2, dvbConfig->devList.at(i)->lnb2 );
-		dvb.at(i)->setLnb( 3, dvbConfig->devList.at(i)->lnb3 );
-	}
 	rtp->setSocket( dvbConfig->broadcastAddress, dvbConfig->broadcastPort, dvbConfig->senderPort );
 	cleaner->setPath( dvbConfig->shiftDir );
 }
@@ -1051,7 +1061,7 @@ void DvbPanel::fillChannelList()
 		}
 		else if ( currentCategory!="All" && chan->category!=currentCategory )
 			continue;
-		it = new KListViewItem( channelsCb, QString().sprintf("%04d", chan->num), chan->name );
+		it = new KListViewItem( channelsCb, QString().sprintf("%04d", chan->num), chan->name, chan->tp.source );
 		it->setDragEnabled( true );
 		if ( !chan->pix.isNull() )
 			it->setPixmap( 1, chan->pix );
@@ -1837,10 +1847,14 @@ void DvbPanel::dvbZap( ChannelDesc *chan )
 	isTuning = true;
 	audioComb->setEnabled( false );
 	emit setTimeShiftFilename( "" );
+	for ( i=0; i<(int)dvb.count(); i++ ) {
+		if ( dvb.at(i)->getCurrentTransponder()==chan->tp ) {
+			d = dvb.at(i);
+			break;
+		}
+	}
 	for ( i=0; i<(int)dvb.count(); i++ ) {
 		if ( dvb.at(i)->hasLive() ) {
-			if ( dvb.at(i)->getLiveChannel().tp==chan->tp )
-				d = dvb.at(i);
 			dvb.at(i)->preStopLive();
 			emit dvbPause( false );
 			dvb.at(i)->stopLive( chan );
@@ -2263,7 +2277,7 @@ bool DvbPanel::getChannelList()
 			}
 
 			chan->pix.load( dvbConfig->dvbConfigIconsDir+chan->name );
-			it = new KListViewItem( channelsCb, QString().sprintf("%04d", chan->num), chan->name );
+			it = new KListViewItem( channelsCb, QString().sprintf("%04d", chan->num), chan->name, chan->tp.source );
 			it->setDragEnabled( true );
 			if ( !chan->pix.isNull() )
 				it->setPixmap( 1, chan->pix );
