@@ -727,7 +727,8 @@ void DVBsi::run()
 				nitEnded = false;
 				ns = new NitSection( &transponders, &nitEnded, &ok, adapter, tuner ); //NIT
 			}
-			getSection( 0x11, 0x42 );         //SDT
+			if ( !getSection( 0x11, 0x42 ) )         //SDT
+				continue;
 			if ( !isRunning ) {
 				out();
 				return;
