@@ -1,19 +1,22 @@
-/***************************************************************************
-                          dvbstream.cpp  -  description
-                             -------------------
-    begin                : Mon Jan 20 2003
-    copyright            : (C) 2003-2005 by Christophe Thommeret
-    email                : hftom@free.fr
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/*
+ * dvbstream.cpp
+ *
+ * Copyright (C) 2003-2006 Christophe Thommeret <hftom@free.fr>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -31,10 +34,12 @@
 #include <netinet/in.h>
 #include <netdb.h>
 #include <errno.h>
+#include <math.h>
 
 #include <linux/dvb/dmx.h>
 
 #include <qdir.h>
+#include <qprogressdialog.h>
 
 #include <klocale.h>
 #include <kapplication.h>
@@ -49,31 +54,28 @@
 
 
 
-DvbStream::DvbStream( int anum, int tnum, fe_type_t type, const QString &src, bool as, const QString &charset )
+DvbStream::DvbStream( Device *d, const QString &charset )
 {
+	dvbDevice = d;
 	isRunning = false;
 	timeShifting = false;
 	waitPause = 0;
 	delOut = 0;
-	adapter = anum;
-	tuner = tnum;
-	setSources( src );
 	fdFrontend = fdDvr = 0;
 	ndmx = 0;
 	currentTransponder = Transponder();
-	frontendName = QString("/dev/dvb/adapter%1/frontend%2").arg( adapter ).arg( tuner );
-	dvrName = QString("/dev/dvb/adapter%1/dvr%2").arg( adapter ).arg( tuner );
-	demuxName = QString("/dev/dvb/adapter%1/demux%2").arg( adapter ).arg( tuner );
-	dvbType = type;
-	canAutoscan = as,
+	frontendName = QString("/dev/dvb/adapter%1/frontend%2").arg( dvbDevice->adapter ).arg( dvbDevice->tuner );
+	dvrName = QString("/dev/dvb/adapter%1/dvr%2").arg( dvbDevice->adapter ).arg( dvbDevice->tuner );
+	demuxName = QString("/dev/dvb/adapter%1/demux%2").arg( dvbDevice->adapter ).arg( dvbDevice->tuner );
 
 	out.setAutoDelete( true );
 
 	bool bok;
-	dvbEvents = new DVBevents( &bok, adapter, tuner, charset );
+	dvbEvents = new DVBevents( &bok, dvbDevice->adapter, dvbDevice->tuner, charset );
 
 	camProbed = false;
 	cam = NULL;
+	plug = NULL;
 
 	connect( &statusTimer, SIGNAL(timeout()), this, SLOT(checkStatus()) );
 }
@@ -92,48 +94,33 @@ void DvbStream::probeCam()
 	int ci_type=0;
 	if ( camProbed )
 		return;
-	if ( (ci_type=DvbCam::probe( adapter, 0 ))>0 )
-		cam = new DvbCam( adapter, 0, tuner, ci_type );
+	if ( (ci_type=DvbCam::probe( dvbDevice->adapter, 0 ))>0 )
+		cam = new DvbCam( dvbDevice->adapter, 0, dvbDevice->tuner, ci_type );
 	camProbed = true;
 }
 
 
 
-void DvbStream::setLnb( int pos, LNB b )
-{
-	lnb[pos] = b;
-}
-
-
-
-void DvbStream::setSources( const QString &src )
-{
-	QString s;
-	int pos, i;
-
-	s = src;
-	source.clear();
-	pos = s.find("|");
-	s = s.right( s.length()-pos-1 );
-	for ( i=0; i<src.contains("|"); i++ ) {
-		pos = s.find("|");
-		source.append( s.left(pos) );
-		s = s.right( s.length()-pos-1 );
-	}
-}
-
-
-
 QStringList DvbStream::getSources( bool all )
 {
 	if ( !all ) {
-		if ( dvbType==FE_OFDM )
+		if ( dvbDevice->type==FE_OFDM )
 			return QStringList( "Terrestrial" );
-		if ( dvbType==FE_QAM )
+		if ( dvbDevice->type==FE_QAM )
 			return QStringList( "Cable" );
-		if ( dvbType==FE_ATSC )
+		if ( dvbDevice->type==FE_ATSC )
 			return QStringList( "Atsc" );
 	}
+	QStringList source;
+	int i;
+	if ( dvbDevice->type==FE_QPSK ) {
+		for ( i=0; i<dvbDevice->numLnb; i++ )
+			source+= dvbDevice->lnb[i].source;
+	}
+	else {
+		QString s = dvbDevice->source;
+		source+= s.remove(0,2);
+	}
 	return source;
 }
 
@@ -141,20 +128,21 @@ QStringList DvbStream::getSources( bool all )
 
 bool DvbStream::canSource( ChannelDesc *chan )
 {
-	if ( chan->tp.type!=dvbType ) {
+	if ( chan->tp.type!=dvbDevice->type ) {
 		return false;
 	}
-	if ( dvbType!=FE_QPSK ) {
-		if ( chan->tp.type==dvbType )
+	if ( dvbDevice->type!=FE_QPSK ) {
+		if ( chan->tp.type==dvbDevice->type )
 			return true;
 		else
 			return false;
 	}
-	int i = getSatPos( chan->tp.source );
-	if ( i!=-1 )
-		return true;
-	else
-		return false;
+	int i;
+	for ( i=0; i<dvbDevice->numLnb; i++ ) {
+		if ( dvbDevice->lnb[i].source.contains(chan->tp.source) )
+			return true;
+	}
+	return false;
 }
 
 
@@ -163,8 +151,11 @@ int DvbStream::getSatPos( const QString &src )
 {
 	int i;
 
-	for ( i=0; i<(int)source.count(); i++ )
-		if ( source[i]==src )
+	if ( dvbDevice->type!=FE_QPSK )
+		return -1;
+
+	for ( i=0; i<dvbDevice->numLnb; i++ )
+		if ( dvbDevice->lnb[i].source.contains(src) )
 			return i;
 	return -1;
 }
@@ -310,6 +301,8 @@ bool DvbStream::tuneDvb( ChannelDesc *chan, bool dvr )
 	unsigned long freq=chan->tp.freq;
 	unsigned long srate=chan->tp.sr;
 	int lnbPos = getSatPos( chan->tp.source );
+	int rotorMove = 0;
+	int loop=0, i;
 
 	closeFe();
 	if ( !openFe() )
@@ -322,7 +315,7 @@ bool DvbStream::tuneDvb( ChannelDesc *chan, bool dvr )
 
 	probeCam();
 
-	fprintf(stderr, "Using DVB device %d:%d \"%s\"\n", adapter, tuner, fe_info.name);
+	fprintf(stderr, "Using DVB device %d:%d \"%s\"\n", dvbDevice->adapter, dvbDevice->tuner, fe_info.name);
 
 	freq*=1000;
 	srate*=1000;
@@ -358,17 +351,24 @@ bool DvbStream::tuneDvb( ChannelDesc *chan, bool dvr )
 		case FE_QPSK : {
 			fprintf(stderr,"tuning DVB-S to %lu %c %lu\n", freq, chan->tp.pol, srate);
 			if (freq > 2200000) {
-				if ( lnb[lnbPos].switchFreq ) {
-					if ( freq<(lnb[lnbPos].switchFreq*1000) ) {
-						lof = lnb[lnbPos].loFreq*1000;
+				if ( dvbDevice->lnb[lnbPos].switchFreq ) { // Dual LO
+					if ( freq<(dvbDevice->lnb[lnbPos].switchFreq*1000) ) {
+						lof = dvbDevice->lnb[lnbPos].loFreq*1000;
 					}
 					else {
-						lof = lnb[lnbPos].hiFreq*1000;
+						lof = dvbDevice->lnb[lnbPos].hiFreq*1000;
 						hiband = 1;
 					}
 				}
 				else {
-					lof = (lnb[lnbPos].loFreq*1000);
+					if ( dvbDevice->lnb[lnbPos].hiFreq ) { // C Band Multipoint
+						if ( (chan->tp.pol=='h') || (chan->tp.pol=='H') )
+							lof = (dvbDevice->lnb[lnbPos].hiFreq*1000);
+						else
+							lof = (dvbDevice->lnb[lnbPos].loFreq*1000);
+					}
+					else // Single LO
+						lof = (dvbDevice->lnb[lnbPos].loFreq*1000);
 				}
 				if ( freq<lof )
 					feparams.frequency = ( lof-freq );
@@ -381,7 +381,7 @@ bool DvbStream::tuneDvb( ChannelDesc *chan, bool dvr )
 			feparams.u.qpsk.symbol_rate=srate;
 			feparams.u.qpsk.fec_inner=chan->tp.coderateH;
 			fprintf(stderr,"inv:%d fecH:%d\n", chan->tp.inversion, chan->tp.coderateH );
-			if ( setupSwitch( lnbPos, (chan->tp.pol=='h') || (chan->tp.pol=='H'), hiband)!=0 ) {
+			if ( setDiseqc( lnbPos, chan, hiband, rotorMove, dvr )!=0 ) {
 				closeFe();
 				return false;
 			}
@@ -406,23 +406,30 @@ bool DvbStream::tuneDvb( ChannelDesc *chan, bool dvr )
 	else
 		feparams.inversion=chan->tp.inversion;
 
-	if (ioctl(fdFrontend,FE_SET_FRONTEND,&feparams) < 0) {
-		perror("ERROR tuning \n");
-		closeFe();
-		return false;
-	}
+	if ( rotorMove )
+		loop = 2;
 
-	for ( int i=0; i<15; i++ ) {
-		usleep( 100000 );
-		fprintf( stderr, "." );
-		if ( ioctl( fdFrontend, FE_READ_STATUS, &status )==-1 ) {
-			perror( "FE_READ_STATUS" );
-			break;
+	while ( loop>-1 ) {
+		if (ioctl(fdFrontend,FE_SET_FRONTEND,&feparams) < 0) {
+			perror("ERROR tuning \n");
+			closeFe();
+			return false;
 		}
-		if ( status & FE_HAS_LOCK ) {
-			fprintf(stderr," LOCKED.\n");
-			break;
+		for ( i=0; i<(dvbDevice->tuningTimeout/100); i++ ) {
+			usleep( 100000 );
+			fprintf( stderr, "." );
+			if ( ioctl( fdFrontend, FE_READ_STATUS, &status )==-1 ) {
+				perror( "FE_READ_STATUS" );
+				break;
+			}
+			if ( status & FE_HAS_LOCK ) {
+				fprintf(stderr," LOCKED.");
+				loop = 0;
+				break;
+			}
 		}
+		fprintf(stderr,"\n");
+		--loop;
 	}
 
 	if ( !( status & FE_HAS_LOCK ) ) {
@@ -431,6 +438,9 @@ bool DvbStream::tuneDvb( ChannelDesc *chan, bool dvr )
 		return false;
 	}
 
+	if ( rotorMove )
+		dvbDevice->lnb[lnbPos].currentSource = chan->tp.source;
+
 	if ( !dvr )
 		return true;
 
@@ -449,73 +459,208 @@ bool DvbStream::tuneDvb( ChannelDesc *chan, bool dvr )
 
 
 
-int DvbStream::setupSwitch( int switchPos, int voltage18, int hiband )
+int DvbStream::setDiseqc( int switchPos, ChannelDesc *chan, int hiband, int &rotor, bool dvr )
 {
-	struct diseqc_cmd switch_cmds[] = {
-		{ { { 0xe0, 0x10, 0x38, 0xf0, 0x00, 0x00 }, 4 }, 0 },
-		{ { { 0xe0, 0x10, 0x38, 0xf2, 0x00, 0x00 }, 4 }, 0 },
-		{ { { 0xe0, 0x10, 0x38, 0xf1, 0x00, 0x00 }, 4 }, 0 },
-		{ { { 0xe0, 0x10, 0x38, 0xf3, 0x00, 0x00 }, 4 }, 0 },
-		{ { { 0xe0, 0x10, 0x38, 0xf4, 0x00, 0x00 }, 4 }, 0 },
-		{ { { 0xe0, 0x10, 0x38, 0xf6, 0x00, 0x00 }, 4 }, 0 },
-		{ { { 0xe0, 0x10, 0x38, 0xf5, 0x00, 0x00 }, 4 }, 0 },
-		{ { { 0xe0, 0x10, 0x38, 0xf7, 0x00, 0x00 }, 4 }, 0 },
-		{ { { 0xe0, 0x10, 0x38, 0xf8, 0x00, 0x00 }, 4 }, 0 },
-		{ { { 0xe0, 0x10, 0x38, 0xfa, 0x00, 0x00 }, 4 }, 0 },
-		{ { { 0xe0, 0x10, 0x38, 0xf9, 0x00, 0x00 }, 4 }, 0 },
-		{ { { 0xe0, 0x10, 0x38, 0xfb, 0x00, 0x00 }, 4 }, 0 },
-		{ { { 0xe0, 0x10, 0x38, 0xfc, 0x00, 0x00 }, 4 }, 0 },
-		{ { { 0xe0, 0x10, 0x38, 0xfe, 0x00, 0x00 }, 4 }, 0 },
-		{ { { 0xe0, 0x10, 0x38, 0xfd, 0x00, 0x00 }, 4 }, 0 },
-		{ { { 0xe0, 0x10, 0x38, 0xff, 0x00, 0x00 }, 4 }, 0 }
+	struct dvb_diseqc_master_cmd switchCmd[] = {
+		{ { 0xe0, 0x10, 0x38, 0xf0, 0x00, 0x00 }, 4 },
+		{ { 0xe0, 0x10, 0x38, 0xf2, 0x00, 0x00 }, 4 },
+		{ { 0xe0, 0x10, 0x38, 0xf1, 0x00, 0x00 }, 4 },
+		{ { 0xe0, 0x10, 0x38, 0xf3, 0x00, 0x00 }, 4 },
+		{ { 0xe0, 0x10, 0x38, 0xf4, 0x00, 0x00 }, 4 },
+		{ { 0xe0, 0x10, 0x38, 0xf6, 0x00, 0x00 }, 4 },
+		{ { 0xe0, 0x10, 0x38, 0xf5, 0x00, 0x00 }, 4 },
+		{ { 0xe0, 0x10, 0x38, 0xf7, 0x00, 0x00 }, 4 },
+		{ { 0xe0, 0x10, 0x38, 0xf8, 0x00, 0x00 }, 4 },
+		{ { 0xe0, 0x10, 0x38, 0xfa, 0x00, 0x00 }, 4 },
+		{ { 0xe0, 0x10, 0x38, 0xf9, 0x00, 0x00 }, 4 },
+		{ { 0xe0, 0x10, 0x38, 0xfb, 0x00, 0x00 }, 4 },
+		{ { 0xe0, 0x10, 0x38, 0xfc, 0x00, 0x00 }, 4 },
+		{ { 0xe0, 0x10, 0x38, 0xfe, 0x00, 0x00 }, 4 },
+		{ { 0xe0, 0x10, 0x38, 0xfd, 0x00, 0x00 }, 4 },
+		{ { 0xe0, 0x10, 0x38, 0xff, 0x00, 0x00 }, 4 },
 	};
-	struct diseqc_cmd *cmd[2] = { NULL, NULL };
-	int i = 4 * switchPos + 2 * hiband + (voltage18 ? 1 : 0);
 
-	fprintf( stderr, "DiSEqC: switch pos %i, %sV, %sband (index %d)\n", switchPos, voltage18 ? "18" : "13", hiband ? "hi" : "lo", i );
-	if ( i < 0 || i >= (int)(sizeof(switch_cmds)/sizeof(struct diseqc_cmd)) )
+	int voltage18 = ( (chan->tp.pol=='H')||(chan->tp.pol=='h') );
+	int ci = 4 * switchPos + 2 * hiband + (voltage18 ? 1 : 0);
+
+	fprintf( stderr, "DiSEqC: switch pos %i, %sV, %sband (index %d)\n", switchPos, voltage18 ? "18" : "13", hiband ? "hi" : "lo", ci );
+	if ( ci < 0 || ci >= (int)(sizeof(switchCmd)/sizeof(struct dvb_diseqc_master_cmd)) )
 		return -EINVAL;
-	cmd[0] = &switch_cmds[i];
-	return diseqc( i % 2 ? SEC_VOLTAGE_18 : SEC_VOLTAGE_13, cmd,
-		(i/2) % 2 ? SEC_TONE_ON : SEC_TONE_OFF, (i/4) % 2 ? SEC_MINI_B : SEC_MINI_A );
+
+	if ( ioctl(fdFrontend, FE_SET_TONE, SEC_TONE_OFF) )
+		perror("FE_SET_TONE failed");
+	if ( ioctl(fdFrontend, FE_SET_VOLTAGE, ci%2 ? SEC_VOLTAGE_18 : SEC_VOLTAGE_13) )
+		perror("FE_SET_VOLTAGE failed");
+	usleep(15*1000);
+	fprintf( stderr, "DiSEqC: %02x %02x %02x %02x %02x %02x\n", switchCmd[ci].msg[0], switchCmd[ci].msg[1], switchCmd[ci].msg[2], switchCmd[ci].msg[3], switchCmd[ci].msg[4], switchCmd[ci].msg[5] );
+	if ( ioctl(fdFrontend, FE_DISEQC_SEND_MASTER_CMD, &switchCmd[ci]) )
+		perror("FE_DISEQC_SEND_MASTER_CMD failed");
+
+	QString msg;
+	if ( dvbDevice->lnb[switchPos].rotorType!=0 && chan->tp.source!=dvbDevice->lnb[switchPos].currentSource ) {
+		int i, index=-1;
+		double angle=0.0, oldAngle=0.0;
+		fprintf( stderr, "Driving rotor to %s\n", chan->tp.source.ascii() );
+		for ( i=0; i<(int)dvbDevice->lnb[switchPos].source.count(); i++ ) {
+			if ( dvbDevice->lnb[switchPos].source[i]==chan->tp.source ) {
+				index = i;
+				break;
+			}
+		}
+		angle = getSourceAngle( chan->tp.source );
+		if ( dvbDevice->lnb[switchPos].rotorType==1 ) {
+			fprintf( stderr, "Rotor: gotoX=%f\n", angle );
+			gotoX( angle );
+		}
+		else {
+			int pos = dvbDevice->lnb[switchPos].position[index];
+			fprintf( stderr, "Rotor: gotoN=%d\n", pos );
+			rotorCommand( 9, pos );
+		}
+		if ( dvbDevice->lnb[switchPos].currentSource.isEmpty() ) {
+			rotor = 10;
+			msg =  i18n("Moving rotor from unknown position...");
+		}
+		else {
+			oldAngle = getSourceAngle( dvbDevice->lnb[switchPos].currentSource );
+			fprintf( stderr, "old rotor pos: %f °\n", oldAngle );
+			fprintf( stderr, "new rotor pos: %f °\n", angle );
+			angle = fabs(angle-oldAngle);
+			fprintf( stderr, "Rotation angle: %f °\n", angle );
+			if ( voltage18 )
+				rotor = (int)(angle*dvbDevice->lnb[switchPos].speed18v)+1;
+			else
+				rotor = (int)(angle*dvbDevice->lnb[switchPos].speed13v)+1;
+			 msg = i18n("Moving rotor...");
+		}
+		fprintf( stderr, "Rotation time: %d sec.\n", rotor );
+	}
+
+	if ( rotor ) {
+		int j;
+		if ( !dvr ) {
+			for ( j=0; j<(rotor*2); j++ ) {
+				usleep( 500000 );
+			}
+		}
+		else {
+			QProgressDialog progress( msg, i18n("Cancel"), rotor*2, 0, "progress", true );
+			for ( j=0; j<(rotor*2); j++ ) {
+				progress.setProgress( j );
+				qApp->processEvents();
+				if ( progress.wasCanceled() )
+					break;
+				usleep( 500000 );
+			}
+			progress.setProgress( rotor*2 );
+			qApp->processEvents();
+		}
+	}
+
+	usleep(15*1000);
+	if ( ioctl(fdFrontend, FE_DISEQC_SEND_BURST, (ci/4)%2 ? SEC_MINI_B : SEC_MINI_A) )
+		perror("FE_DISEQC_SEND_BURST failed");
+	usleep(15*1000);
+	if ( ioctl(fdFrontend, FE_SET_TONE, (ci/2)%2 ? SEC_TONE_ON : SEC_TONE_OFF) )
+		perror("FE_SET_TONE failed");
+
+	return 0;
 }
 
 
 
-int DvbStream::diseqc( fe_sec_voltage_t v, struct diseqc_cmd **cmd, fe_sec_tone_mode_t t, fe_sec_mini_cmd_t b)
+double DvbStream::getSourceAngle( QString source )
 {
-	int err;
+	double angle=1.0;
+	int pos = source.findRev("-");
+	source.remove(0,pos+1);
+	source = source.upper();
+	if ( source.endsWith("W") )
+		angle = -1.0;
+	source.truncate( source.length()-1 );
+	angle*= source.toDouble();
+	return getAzimuth( angle );
+}
 
-	if ((err = ioctl(fdFrontend, FE_SET_TONE, SEC_TONE_OFF))) {
-		perror("FE_SET_TONE failed");
-		//return err;
+
+
+#define TO_RADS (M_PI / 180.0)
+#define TO_DEC (180.0 / M_PI)
+
+double DvbStream::getAzimuth( double angle )
+{
+	double latitude = dvbDevice->usalsLatitude;
+	double longitude = dvbDevice->usalsLongitude;
+	double P, Ue, Us, az, x, el, Azimuth;
+
+	P = latitude*TO_RADS;           // Earth Station Latitude
+	Ue = longitude*TO_RADS;           // Earth Station Longitude
+	Us = angle*TO_RADS;          // Satellite Longitude
+
+	az = M_PI+atan( tan( Us-Ue )/sin( P ) );
+	x = acos( cos(Us-Ue)*cos(P) );
+	el = atan( ( cos( x )-0.1513 )/sin( x ) );
+	Azimuth = atan( ( -cos(el)*sin(az) )/( sin(el)*cos(P)-cos(el)*sin(P)*cos(az)) )* TO_DEC;
+
+	return Azimuth;
+}
+
+
+
+void DvbStream::gotoX( double azimuth )
+{
+	double USALS=0.0;
+	int CMD1=0x00, CMD2=0x00;
+	int DecimalLookup[10] = { 0x00, 0x02, 0x03, 0x05, 0x06, 0x08, 0x0A, 0x0B, 0x0D, 0x0E };
+
+	if ( azimuth>0.0 )
+		CMD1 = 0xE0;    // East
+	else
+		CMD1 = 0xD0;      // West
+
+	USALS = fabs( azimuth );
+
+	while (USALS > 16) {
+		CMD1++;
+		USALS-= 16;
 	}
-	if ((err = ioctl(fdFrontend, FE_SET_VOLTAGE, v))) {
-		perror("FE_SET_VOLTAGE failed");
-		//return err;
+	while (USALS >= 1.0) {
+		CMD2+=0x10;
+		USALS--;
 	}
-	usleep(15*1000);
-	while (*cmd) {
-		fprintf( stderr, "DiSEqC: %02x %02x %02x %02x %02x %02x\n", (*cmd)->cmd.msg[0], (*cmd)->cmd.msg[1],
-			(*cmd)->cmd.msg[2], (*cmd)->cmd.msg[3], (*cmd)->cmd.msg[4], (*cmd)->cmd.msg[5]);
-		if ((err = ioctl(fdFrontend, FE_DISEQC_SEND_MASTER_CMD, &(*cmd)->cmd))) {
-			perror("FE_DISEQC_SEND_MASTER_CMD failed");
-			//return err;
-		}
-		usleep( (*cmd)->wait*1000 );
-		cmd++;
-	}
-	usleep(15*1000);
-	if ((err = ioctl(fdFrontend, FE_DISEQC_SEND_BURST, b))) {
-		perror("FE_DISEQC_SEND_BURST failed");
-		//return err;
-	}
-	usleep(15*1000);
-	if ((err = ioctl(fdFrontend, FE_SET_TONE, t))) {
-		perror("FE_SET_TONE failed");
-		//return err;
+	USALS*= 10.0;
+	int rd = (int)USALS;
+	USALS-= rd;
+	if ( USALS>0.5 )
+		++rd;
+	CMD2+= DecimalLookup[rd];
+
+	rotorCommand( 12, CMD1, CMD2 );
+}
+
+
+
+void DvbStream::rotorCommand( int cmd, int n1, int n2, int n3 )
+{
+	struct dvb_diseqc_master_cmd cmds[] = {
+		{ { 0xe0, 0x31, 0x60, 0x00, 0x00, 0x00 }, 3 },  //0 Stop Positioner movement
+		{ { 0xe0, 0x31, 0x63, 0x00, 0x00, 0x00 }, 3 },  //1 Disable Limits
+		{ { 0xe0, 0x31, 0x66, 0x00, 0x00, 0x00 }, 3 },  //2 Set East Limit
+		{ { 0xe0, 0x31, 0x67, 0x00, 0x00, 0x00 }, 3 },  //3 Set West Limit
+		{ { 0xe0, 0x31, 0x68, 0x00, 0x00, 0x00 }, 4 },  //4 Drive Motor East continously
+		{ { 0xe0, 0x31, 0x68,256-n1,0x00, 0x00 }, 4 },  //5 Drive Motor East nn steps
+		{ { 0xe0, 0x31, 0x69,256-n1,0x00, 0x00 }, 4 },  //6 Drive Motor West nn steps
+		{ { 0xe0, 0x31, 0x69, 0x00, 0x00, 0x00 }, 4 },  //7 Drive Motor West continously
+		{ { 0xe0, 0x31, 0x6a, n1, 0x00, 0x00 }, 4 },  //8 Store nn
+		{ { 0xe0, 0x31, 0x6b, n1, 0x00, 0x00 }, 4 },   //9 Goto nn
+		{ { 0xe0, 0x31, 0x6f, n1, n2, n3 }, 4}, //10 Recalculate Position
+		{ { 0xe0, 0x31, 0x6a, 0x00, 0x00, 0x00 }, 4 },  //11 Enable Limits
+		{ { 0xe0, 0x31, 0x6e, n1, n2, 0x00 }, 5 },   //12 Gotoxx
+		{ { 0xe0, 0x10, 0x38, 0xF4, 0x00, 0x00 }, 4 }    //13 User
 	};
-	return 0;
+
+	if ( ioctl( fdFrontend, FE_DISEQC_SEND_MASTER_CMD, &cmds[cmd] ) )
+		perror("Rotor : FE_DISEQC_SEND_MASTER_CMD failed");
 }
 
 
@@ -619,9 +764,12 @@ bool DvbStream::checkStatus()
 
 int DvbStream::getSNR()
 {
-	int32_t snr;
-	ioctl( fdFrontend, FE_READ_SNR, &snr );
-	return snr*100/65535;
+	int32_t snr=0;
+	if ( fdFrontend ) {
+		ioctl( fdFrontend, FE_READ_SNR, &snr );
+		snr = snr*100/65535;
+	}
+	return snr;
 }
 
 
@@ -733,6 +881,8 @@ void DvbStream::receivePlayDvb()
 	KApplication::kApplication()->postEvent( this, new QTimerEvent( 500 ) );
 }
 
+
+
 void DvbStream::timerEvent( QTimerEvent* ev )
 {
 	switch ( ev->timerId() ) {
@@ -876,7 +1026,7 @@ bool DvbStream::startBroadcast( QPtrList<ChannelDesc> *list, Ts2Rtp *rtp )
 		}
 
 		if ( !o ) {
-			o = new DVBout( *list->at(i), adapter, tuner, plug );
+			o = new DVBout( *list->at(i), dvbDevice->adapter, dvbDevice->tuner, plug );
 			connect( o, SIGNAL(endRecord(DVBout*,RecTimer*,bool)), this, SLOT(recordEnded(DVBout*,RecTimer*,bool)) );
 			connect( o, SIGNAL(playDvb()), this, SLOT(receivePlayDvb()) );
 			connect( o, SIGNAL(shifting(bool)), this, SLOT(receiveShifting(bool)) );
@@ -991,7 +1141,7 @@ bool DvbStream::startTimer( ChannelDesc *chan, QString path, RecTimer *t )
 	}
 
 	if ( !o ) {
-		o = new DVBout( *chan, adapter, tuner, plug );
+		o = new DVBout( *chan, dvbDevice->adapter, dvbDevice->tuner, plug );
 		connect( o, SIGNAL(endRecord(DVBout*,RecTimer*,bool)), this, SLOT(recordEnded(DVBout*,RecTimer*,bool)) );
 		connect( o, SIGNAL(playDvb()), this, SLOT(receivePlayDvb()) );
 		connect( o, SIGNAL(shifting(bool)), this, SLOT(receiveShifting(bool)) );
@@ -1066,7 +1216,7 @@ int DvbStream::goLive( ChannelDesc *chan, const QString &pipeName, int &napid )
 	}
 
 	if ( !o ) {
-		o = new DVBout( *chan, adapter, tuner, plug );
+		o = new DVBout( *chan, dvbDevice->adapter, dvbDevice->tuner, plug );
 		connect( o, SIGNAL(endRecord(DVBout*,RecTimer*,bool)), this, SLOT(recordEnded(DVBout*,RecTimer*,bool)) );
 		connect( o, SIGNAL(playDvb()), this, SLOT(receivePlayDvb()) );
 		connect( o, SIGNAL(shifting(bool)), this, SLOT(receiveShifting(bool)) );
