@@ -1,20 +1,22 @@
-/***************************************************************************
-                          dvbstream.h  -  description
-                             -------------------
-    begin                : Mon Jan 20 2003
-    copyright            : (C) 2003-2005 by Christophe Thommeret
-    email                : hftom@free.fr
-    last modified        : $Date: 2005/08/06 23:28:07 $ by $Author: hftom $
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/*
+ * dvbstream.h
+ *
+ * Copyright (C) 2003-2006 Christophe Thommeret <hftom@free.fr>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+ */
 
 #ifndef DVBSTREAM_H
 #define DVBSTREAM_H
@@ -42,13 +44,6 @@ class KaffeineDvbPlugin;
 
 
 
-struct diseqc_cmd {
-	struct dvb_diseqc_master_cmd cmd;
-	uint32_t wait;
-};
-
-
-
 class DvbStream : public QObject, public QThread
 {
 	Q_OBJECT
@@ -58,11 +53,12 @@ public :
 	enum { ErrDontSwitchAudio=-1, ErrCantTune=1, ErrCantSetPids=2, ErrIsRecording=3, ErrNoLive=4,
 		ErrCantOpenFile=5, ErrIsBroadcasting=6, ErrCamUsed=7 };
 
-	DvbStream( int anum, int tnum, fe_type_t type, const QString &src, bool as, const QString &charset );
+	DvbStream( Device *d, const QString &charset );
 	~DvbStream();
+	int getAdapter() { return dvbDevice->adapter; }
+	int getTuner() { return dvbDevice->tuner; }
+	fe_type_t getType() {return dvbDevice->type; }
 	void setPlug( KaffeineDvbPlugin *p );
-	void setSources( const QString &src );
-	void setLnb( int pos, LNB b );
 	QStringList getSources( bool all=false );
 	bool canSource( ChannelDesc *chan );
 	bool tuneDvb( ChannelDesc *chan, bool dvr=true );
@@ -95,9 +91,6 @@ public :
 	unsigned char thBuf[188*10];
 	struct pollfd pfd;
 	DVBevents *dvbEvents;
-	int adapter, tuner;
-	fe_type_t dvbType;
-	bool canAutoscan;
 
 public slots:
 
@@ -113,8 +106,11 @@ protected:
 
 private :
 
-	int setupSwitch( int switchPos, int voltage18, int hiband );
-	int diseqc( fe_sec_voltage_t v, struct diseqc_cmd **cmd, fe_sec_tone_mode_t t, fe_sec_mini_cmd_t b);
+	int setDiseqc( int switchPos, ChannelDesc *chan, int hiband, int &rotor, bool dvr );
+	void rotorCommand( int cmd, int n1=0, int n2=0, int n3=0 );
+	void gotoX( double azimuth );
+	double getAzimuth( double angle );
+	double getSourceAngle( QString source );
 	int getSatPos( const QString &src );
 	bool setPids( DVBout *o, int napid=0 );
 	void removePids( DVBout *o );
@@ -122,14 +118,14 @@ private :
 	void recordingState();
 	void startReading();
 	void probeCam();
+	bool openFe();
+	bool closeFe();
+	void connectStatus( bool con );
 
 	QFile liveFile;
 	bool timeShifting;
 	QPtrList<DVBout> out;
-	LNB lnb[4];
-	bool openFe();
-	bool closeFe();
-	void connectStatus( bool con );
+	Device *dvbDevice;
 	int waitPause;
 	DVBout *delOut;
 	Transponder currentTransponder;
@@ -143,7 +139,6 @@ private :
 	bool isRunning;
 	QTimer statusTimer;
 	QString timeShiftFileName;
-	QStringList source;
 	DvbCam *cam;
 	bool camProbed;
 	KaffeineDvbPlugin *plug;
@@ -163,4 +158,4 @@ signals:
 
 };
 
-#endif
+#endif /* DVBSTREAM_H */
