@@ -30,37 +30,34 @@
 #include "kaffeine.h"
 #include "version.h"
 
-#include <X11/Xlib.h>
+#ifndef HAVE_XCB
+/* including the Xlib header pollutes namespace too much --pfister */
+extern "C" int XInitThreads();
+#endif
 
 class KaffeineApp : public KUniqueApplication
 {
 public:
 	KaffeineApp() : kaffeine(NULL) { }
+
 	~KaffeineApp()
 	{
 		delete kaffeine;
 	}
 
-	int newInstance();
-
 private:
 	Kaffeine *kaffeine;
 
+	int newInstance();
+
 	KaffeineApp(const KaffeineApp &);
-	KaffeineApp operator=(const KaffeineApp &);
+	KaffeineApp &operator=(const KaffeineApp &);
 };
 
 int KaffeineApp::newInstance()
 {
-	/*
-	 * FIXME: session management make lot of troubles; because kaffeine starts more than one process?
-	 * disabled for the moment ...
-	 */
-	if (isRestored())
-		return 0;
-
 	if (kaffeine)
-		kaffeine->processArgs();
+		kaffeine->updateArgs();
 	else
 		kaffeine = new Kaffeine();
 
@@ -70,16 +67,17 @@ int KaffeineApp::newInstance()
 int main(int argc, char *argv[])
 {
 	KAboutData aboutData("kaffeine", I18N_NOOP("Kaffeine Player"), KAFFEINE_VERSION,
-			     I18N_NOOP("A media player for KDE 3. Can use multiple backends for playback, default (and recommended) is xine."),
-			     KAboutData::License_GPL_V2, I18N_NOOP("(c) 2003-2005, The Kaffeine Authors"), 0,
-			     "http://kaffeine.sourceforge.net", "kaffeine-user@lists.sourceforge.net");
+		I18N_NOOP("A media player for KDE 3. Can use multiple backends for playback, default (and recommended) is xine."),
+		KAboutData::License_GPL_V2, I18N_NOOP("Copyright (C) 2003-2006, The Kaffeine Authors"), 0,
+		"http://kaffeine.sourceforge.net");
 
 	aboutData.addAuthor("Christophe Thommeret", I18N_NOOP("Current maintainer"), "hftom@free.fr");
+	aboutData.addAuthor("Christoph Pfister", I18N_NOOP("Developer"), "christophpfister@gmail.com");
 	aboutData.addAuthor("Jürgen Kofler", I18N_NOOP("Original author"), "kaffeine@gmx.net");
 
 	/* FIXME: what about the other contributors listed in CREDITS? --pfister */
 
-	aboutData.addCredit("Christoph Pfister", I18N_NOOP("CICAM support."), "christophpfister@gmail.com");
+	aboutData.addCredit("Ricardo Manuel Santos Rodrigues", I18N_NOOP("Various patches."), "madinfo@cadaval.net");
 	aboutData.addCredit("Christopher Martin", I18N_NOOP("Various valuable patches."), "christopher.martin@utoronto.ca");
 	aboutData.addCredit("Ben Jackson", I18N_NOOP("DVB OSD browsing patch."), "benj@puremourning.co.uk");
 	aboutData.addCredit("Rainer Wirtz", I18N_NOOP("DVB categories patches."), "rainer.wirtz@gmx.de");
@@ -96,6 +94,7 @@ int main(int argc, char *argv[])
 	if (!KaffeineApp::start())
 		return 0;
 
+#ifndef HAVE_XCB
 	/*
 	 * XInitThreads() should the first call to xlib in multithreaded X programs -
 	 * but because of a bug in some xfree versions that can freeze at startup,
@@ -111,6 +110,7 @@ int main(int argc, char *argv[])
 	flush(kdDebug() << "if kaffeine hangs here run 'configure --with-xinit-workaround' and recompile / reinstall ...");
 	XInitThreads();
 	kdDebug() << " ok\n";
+#endif
 #endif
 
 	KaffeineApp a;
