@@ -1,22 +1,22 @@
-/***************************************************************************
- *   Copyright (C) 2004-2005 by Jürgen Kofler                                   *
- *   kaffeine@gmx.net                                                      *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
- ***************************************************************************/
+/*
+ * xine_part.cpp
+ *
+ * Copyright (C) 2004-2005 Jürgen Kofler <kaffeine@gmx.net>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+ */
 
 #include "xine_part.h"
 
@@ -68,7 +68,7 @@ K_EXPORT_COMPONENT_FACTORY (libxinepart, XinePartFactory)
 
 XinePart::XinePart(QWidget* parentWidget, const char* widgetName, QObject* parent, const char* name, const QStringList& args)
 		: DCOPObject("XinePartIface"),
-		KaffeinePart(parent, name ? name : "XinePart"), m_hrefParam(QString::null), m_autoStart(true), m_imageWindow(true),
+		KaffeinePart(parent, name ? name : "XinePart"),
 		m_current(0), m_xine(NULL), m_pictureSettings(NULL), m_deinterlacerConfigWidget(NULL),
 		m_filterDialog(NULL), m_embeddedContext(NULL)
 {
@@ -77,8 +77,6 @@ XinePart::XinePart(QWidget* parentWidget, const char* widgetName, QObject* paren
 	/*
 	 *   Parsing parameter given by kaffeine (audiodriver, videodriver, verbose)
 	 *   or parameters of <embed>...</embed>
-	 *   Fix for Quicktime movies on www.apple.com:
-	 *   use the given href parameter instead of the URL passed by konqueror
 	 *
 	 *   format: param="value"
 	 */
@@ -109,72 +107,40 @@ XinePart::XinePart(QWidget* parentWidget, const char* widgetName, QObject* paren
 				verbose = true;
 			}
 		}
-
-		if (args[i].left(9).lower()  == "autostart")
-		{
-			if (args[i].section( '"', 1, 1 ).lower() == "false")
-			{
-				kdDebug() << "XinePart: Found parameter autoStart=false, disable autostart" << endl;
-				m_autoStart = false;
-			}
-		}
-		if (args[i].left(4).lower() == "href")
-		{
-			m_hrefParam = args[i].section( '"',1, 1 );
-			kdDebug() << "XinePart: Found href parameter, value: " << m_hrefParam << endl;
-		}
-		if (args[i].left(8).lower() == "controls")
-		{
-			if (args[i].section( '"', 1, 1 ).lower() != "imagewindow")
-			{
-				kdDebug() << "XinePart: Not an ImageWindow object" << endl;
-				m_imageWindow = false;
-			}
-		}
 	}
 
 	// we need an instance
 	setInstance(XinePartFactory::instance());
 
-	// Do not load everything if nothing is to be played
-	if (m_imageWindow)
-	{
-		QString configPath = locateLocal("appdata", "xine-config");
-		//maybe we are inside konqueror or ark
-		if (!configPath.contains("kaffeine"))
-			configPath = locate("data", "kaffeine/xine-config");
-		kdDebug() << "XinePart: Using xine-config file: " << configPath << endl;
+	// be careful - we may be embedded
+	QString configPath = locate("data", "kaffeine/xine-config");
+	QString logoPath = locate("data", "kaffeine/logo");
 
-		QString logoPath = locate("appdata", "logo");
-		if (!logoPath)
-			logoPath = locate("data", "kaffeine/logo");
+	kdDebug() << "XinePart: Using xine-config file: " << configPath << endl;
 
-		m_xine = new KXineWidget(parentWidget, widgetName, configPath, logoPath,
-		                         audioDriver, videoDriver, /* start xine manual*/true, verbose);
-		connect(m_xine, SIGNAL(signalXineFatal(const QString&)), this, SIGNAL(canceled(const QString&)));
-		connect(m_xine, SIGNAL(stopDvb()), this, SIGNAL(stopDvb()));
-		connect(m_xine, SIGNAL(signalDvbOSDHidden()), this, SIGNAL(dvbOSDHide()));
-		m_xine->setFocusPolicy(QWidget::ClickFocus);
-		setWidget(m_xine);
+	m_xine = new KXineWidget(parentWidget, widgetName, configPath, logoPath,
+	                         audioDriver, videoDriver, /* start xine manual*/true, verbose);
+	connect(m_xine, SIGNAL(signalXineFatal(const QString&)), this, SIGNAL(canceled(const QString&)));
+	connect(m_xine, SIGNAL(stopDvb()), this, SIGNAL(stopDvb()));
+	connect(m_xine, SIGNAL(signalDvbOSDHidden()), this, SIGNAL(dvbOSDHide()));
+	m_xine->setFocusPolicy(QWidget::ClickFocus);
+	setWidget(m_xine);
 
-		// set our XML-UI resource file
-		setXMLFile("xine_part.rc");
-		initActions();
-		initConnections();
+	// set our XML-UI resource file
+	setXMLFile("xine_part.rc");
+	initActions();
+	initConnections();
 
-		QTimer::singleShot(0, this, SLOT(slotDisableAllActions()));
-		m_oldPosition = m_xine->mapToGlobal(QPoint(0,0));
-		m_posCheckTimer.start(333);
-	}
+	QTimer::singleShot(0, this, SLOT(slotDisableAllActions()));
+	m_oldPosition = m_xine->mapToGlobal(QPoint(0,0));
+	m_posCheckTimer.start(333);
 }
 
 XinePart::~XinePart()
 {
 	kdDebug() << "XinePart: destructor" << endl;
-	if(m_imageWindow) {
-		kdDebug() << "XinePart destructor: calling saveConfig()" << endl;
-		saveConfig();
-	}
+	kdDebug() << "XinePart destructor: calling saveConfig()" << endl;
+	saveConfig();
 	if (m_embeddedContext)
 		delete m_embeddedContext;
 }
@@ -184,8 +150,7 @@ KAboutData *XinePart::createAboutData()
 	KAboutData* aboutData = new KAboutData( "kaffeine", I18N_NOOP("XinePart"),
 	                                        KAFFEINE_VERSION, I18N_NOOP("A xine based player part for Kaffeine."),
 	                                        KAboutData::License_GPL,
-	                                        "(c) 2003-2004, Jürgen Kofler.", 0, "http://kaffeine.sourceforge.net",
-	                                        "kaffeine-user@lists.sourceforge.net");
+	                                        "(c) 2003-2004, Jürgen Kofler.", 0, "http://kaffeine.sourceforge.net");
 	aboutData->addAuthor("Jürgen Kofler.",0, "kaffeine@gmx.net");
 
 	return aboutData;
@@ -198,10 +163,7 @@ bool XinePart::openURL(const MRL& mrl)
 	//     if (!mrl.kurl().isValid())
 	//         return false;
 
-	if (m_hrefParam.isNull())
-		m_mrl = mrl;
-	else
-		m_mrl = m_hrefParam;
+	m_mrl = mrl;
 	m_playlist.clear();
 	m_current = 0;
 	bool playlist = false;
@@ -294,10 +256,7 @@ bool XinePart::openURL(const MRL& mrl)
 		m_playlist.append(m_mrl);
 	}
 
-	if (m_autoStart)
-	{
-		slotPlay(true);
-	}
+	slotPlay(true);
 
 	return true;
 }
@@ -315,10 +274,6 @@ bool XinePart::closeURL()
 void XinePart::slotPlay(bool forcePlay)
 {
 	kdDebug() << "XinePart::slotPlay()" << endl;
-	if(!m_imageWindow) {
-		kdDebug() << "XinePart::slotPlay() : not initialized, exiting" << endl;
-		return;
-	}
 
 	m_pauseButton->setChecked(false);
 	if (m_xine->isPlaying())
@@ -385,11 +340,6 @@ void XinePart::slotPlay(bool forcePlay)
 
 void XinePart::slotStop()
 {
-	if(!m_imageWindow) {
-		kdDebug() << "XinePart::slotStop() : not initialized, exiting" << endl;
-		return;
-	}
-
 	if (!m_xine->isXineReady())
 		return;
 
@@ -598,6 +548,20 @@ void XinePart::slotJumpIncrement(int increment)
 	}
 }
 
+void XinePart::slotAdvanceSubTitle()
+{
+	int spuOffset;
+	m_xine->getspuOffset(spuOffset);
+	m_xine->slotSetSpuOffset(spuOffset+45000);
+}
+
+void XinePart::slotDelaySubTitle()
+{
+	int spuOffset;
+	m_xine->getspuOffset(spuOffset);
+	m_xine->slotSetSpuOffset(spuOffset-45000);
+}
+
 void XinePart::slotSaveStream()
 {
 	if (m_mrl.isEmpty())
@@ -627,7 +591,7 @@ void XinePart::slotSetSubtitle(int channel)
 	else
 	{
 		m_playlist[m_current].setCurrentSubtitle(channel - 1);
-		emit signalNewMeta();
+		emit signalNewMeta(m_mrl);
 		m_xine->savePosition(m_xine->getPosition()-200);
 		slotPlay(true); //force load of new subtitle
 	}
@@ -635,6 +599,31 @@ void XinePart::slotSetSubtitle(int channel)
 	m_xine->showOSDMessage(i18n("Subtitle") + ": " + m_subtitles->items()[channel], DEFAULT_OSD_DURATION);
 }
 
+void XinePart::slotAddSubtitle(void)
+{
+	QString subtitleURL = KFileDialog::getOpenURL(m_mrl.kurl().directory(),
+                         i18n("*.smi *.srt *.sub *.txt *.ssa *.asc|Subtitle Files\n*.*|All Files"),
+                         0, i18n("Select Subtitle File")).path();
+
+	if (!(subtitleURL.isEmpty()))
+	{
+		if (!m_playlist[m_current].subtitleFiles().contains(subtitleURL))
+		{
+			m_playlist[m_current].addSubtitleFile(subtitleURL);
+		}
+
+		int subchannel = m_playlist[m_current].subtitleFiles().size() - 1;
+		m_playlist[m_current].setCurrentSubtitle(subchannel);
+		emit signalNewMeta(m_mrl);
+		m_xine->savePosition(m_xine->getPosition()-200);
+		slotPlay(true); //force load of new subtitle
+
+		emit setStatusBarText(i18n("Subtitle") + ": " + m_subtitles->items()[subchannel]);
+		m_xine->showOSDMessage(i18n("Subtitle") + ": " + m_subtitles->items()[subchannel], DEFAULT_OSD_DURATION);
+	}
+
+}
+
 void XinePart::slotSetAudioChannel(int channel)
 {
 	m_xine->slotSetAudioChannel(channel);
@@ -648,7 +637,9 @@ void XinePart::slotSetDVDTitle(const QString& titleStr)
 	uint title = titleStr.toInt(&ok);
 	if (ok && title > 0 && title <= m_xine->getDVDTitleCount())
 	{
-		m_playlist[m_current] = MRL("dvd://" + QString::number(title));
+		KURL url = m_mrl.kurl();
+		url.addPath(QString::number(title));
+		m_playlist[m_current] = MRL(url);
 		slotPlay(true);
 	}
 }
@@ -669,8 +660,10 @@ void XinePart::slotSetDVDAngle(const QString& angleStr)
 	{
 		uint title = m_xine->currentDVDTitleNumber();
 		uint chapter = m_xine->currentDVDChapterNumber();
+		KURL url = m_mrl.kurl();
 
-		m_playlist[m_current] = MRL("dvd://" + QString::number(title) + "." + QString::number(chapter) + "." + QString::number(angle));
+		url.addPath(QString::number(title) + "." + QString::number(chapter) + "." + QString::number(angle));
+		m_playlist[m_current] = MRL(url);
 		slotPlay(true);
 	}
 }
@@ -680,8 +673,10 @@ void XinePart::setDVDChapter(uint chapter)
 	if (chapter > 0 && chapter <= m_xine->getDVDChapterCount())
 	{
 		uint title = m_xine->currentDVDTitleNumber();
+		KURL url = m_mrl.kurl();
 
-		m_playlist[m_current] = MRL("dvd://" + QString::number(title) + "." + QString::number(chapter));
+		url.addPath(QString::number(title) + "." + QString::number(chapter));
+		m_playlist[m_current] = MRL(url);
 		slotPlay(true);
 	}
 }
@@ -748,7 +743,8 @@ void XinePart::slotNewPosition(int pos, const QTime& playtime)
 	QTime length = m_xine->getLength();
 	QTime calcLength;
 
-	if (!m_xine->isSeekable() || length.isNull() || length < playtime)
+	//if (!m_xine->isSeekable() || length.isNull() || length < playtime)
+	if (!m_xine->isSeekable() )
 	{
 		m_position->setPosition(0,false);
 		m_position->setEnabled(false);
@@ -846,6 +842,38 @@ void XinePart::slotDeinterlaceQuality()
 	delete deinterlaceQuality;
 }
 
+void XinePart::slotSetHue( int i )
+{
+	m_hue = i;
+	if ( i==-1 )
+		return;
+	m_xine->slotSetHue( i );
+}
+
+void XinePart::slotSetSaturation( int i )
+{
+	m_saturation = i;
+	if ( i==-1 )
+		return;
+	m_xine->slotSetSaturation( i );
+}
+
+void XinePart::slotSetContrast( int i )
+{
+	m_contrast = i;
+	if ( i==-1 )
+		return;
+	m_xine->slotSetContrast( i );
+}
+
+void XinePart::slotSetBrightness( int i )
+{
+	m_brightness = i;
+	if ( i==-1 )
+		return;
+	m_xine->slotSetBrightness( i );
+}
+
 void XinePart::slotPictureSettings()
 {
 	if (!m_pictureSettings)
@@ -853,10 +881,10 @@ void XinePart::slotPictureSettings()
 		int hue, sat, contrast, bright, avOffset, spuOffset;
 		m_xine->getVideoSettings(hue, sat, contrast, bright, avOffset, spuOffset);
 		m_pictureSettings = new VideoSettings(hue, sat, contrast, bright, avOffset, spuOffset);
-		connect(m_pictureSettings, SIGNAL(signalNewHue(int)), m_xine, SLOT(slotSetHue(int)));
-		connect(m_pictureSettings, SIGNAL(signalNewSaturation(int)), m_xine, SLOT(slotSetSaturation(int)));
-		connect(m_pictureSettings, SIGNAL(signalNewContrast(int)), m_xine, SLOT(slotSetContrast(int)));
-		connect(m_pictureSettings, SIGNAL(signalNewBrightness(int)), m_xine, SLOT(slotSetBrightness(int)));
+		connect(m_pictureSettings, SIGNAL(signalNewHue(int)), this, SLOT(slotSetHue(int)));
+		connect(m_pictureSettings, SIGNAL(signalNewSaturation(int)), this, SLOT(slotSetSaturation(int)));
+		connect(m_pictureSettings, SIGNAL(signalNewContrast(int)), this, SLOT(slotSetContrast(int)));
+		connect(m_pictureSettings, SIGNAL(signalNewBrightness(int)), this, SLOT(slotSetBrightness(int)));
 		connect(m_pictureSettings, SIGNAL(signalNewAVOffset(int)), m_xine, SLOT(slotSetAVOffset(int)));
 		connect(m_pictureSettings, SIGNAL(signalNewSpuOffset(int)), m_xine, SLOT(slotSetSpuOffset(int)));
 	}
@@ -1058,7 +1086,7 @@ void XinePart::slotTrackPlaying()
 	if (mrl.url() == m_mrl.url())
 	{
 		m_mrl = mrl;
-		emit signalNewMeta();
+		emit signalNewMeta(m_mrl);
 	}
 
 	caption = mrl.title();
@@ -1085,13 +1113,13 @@ void XinePart::slotPlaybackFinished()
 void XinePart::slotNewLength()
 {
 	m_mrl.setLength(m_xine->getLength());
-	emit signalNewMeta();
+	emit signalNewMeta(m_mrl);
 }
 
 void XinePart::slotNewTitle()
 {
 	m_mrl.setTitle(m_xine->getTitle());
-	emit signalNewMeta();
+	emit signalNewMeta(m_mrl);
 	emit setWindowCaption(m_mrl.title());
 }
 
@@ -1190,6 +1218,7 @@ void XinePart::slotInfo()
 	if (m_xine->hasSaveURL())
 		ts << "<tr><td><b>" << i18n("Save Stream as") << ":</b></td><td> " << m_xine->getSaveURL() << "</td></tr>";
 
+	ts << "<tr><td></td><td></td></tr>"; // added for better layout
 	ts << "</table></qt>";
 	KMessageBox::information(0, info, i18n("Track info") );
 }
@@ -1216,6 +1245,8 @@ void XinePart::slotFinalize()
 		actionCollection()->action("player_play")->plug(m_embeddedContext);
 		actionCollection()->action("player_pause")->plug(m_embeddedContext);
 		actionCollection()->action("player_stop")->plug(m_embeddedContext);
+		actionCollection()->action("volume_increase")->plug(m_embeddedContext);
+		actionCollection()->action("volume_decrease")->plug(m_embeddedContext);
 		actionCollection()->action("audio_mute")->plug(m_embeddedContext);
 		m_embeddedContext->insertSeparator();
 		actionCollection()->action("player_track_info")->plug(m_embeddedContext);
@@ -1301,6 +1332,7 @@ void XinePart::initActions()
 
 	m_audioChannels = new KSelectAction(i18n("Audio Channel"), 0, actionCollection(), "audio_channels");
 	m_audioChannels->setToolTip(i18n("Select audio channel"));
+	m_audioChannels->setComboWidth( 50 );
 	connect(m_audioChannels, SIGNAL(activated(int)), this, SLOT(slotSetAudioChannel(int )));
 	m_audioVisual = new KSelectAction(i18n("Audio &Visualization"), 0, actionCollection(), "audio_visualization");
 	connect(m_audioVisual, SIGNAL(activated(const QString&)), m_xine, SLOT(slotSetVisualPlugin(const QString&)));
@@ -1326,9 +1358,14 @@ void XinePart::initActions()
 	new KAction(i18n("&Video Settings"), "configure", Key_V, this, SLOT(slotPictureSettings()), actionCollection(), "video_picture");
 	new KAction(i18n("&Equalizer"), NULL, Key_E, this, SLOT(slotEqualizer()), actionCollection(), "equalizer");
 
+
 	m_subtitles = new KSelectAction(i18n("Subtitle"), 0, actionCollection(), "player_subtitles");
 	m_subtitles->setToolTip(i18n("Select Subtitle"));
+	m_subtitles->setComboWidth( 50 );
 	connect(m_subtitles, SIGNAL(activated(int)), this, SLOT(slotSetSubtitle(int)));
+	new KAction(i18n("Delay Subtitle"), 0, CTRL|ALT|Key_Left, this, SLOT(slotDelaySubTitle()), actionCollection(), "adv_sub");
+	new KAction(i18n("Advance Subtitle"), 0, CTRL|ALT|Key_Right, this, SLOT(slotAdvanceSubTitle()), actionCollection(), "delay_sub");
+	new KAction(i18n("Add subtitle..."), 0, 0, this, SLOT(slotAddSubtitle()), actionCollection(), "add_subtitle");
 
 	new KAction(i18n("&Menu Toggle"), "view_detailed", Key_D, m_xine, SLOT(slotMenuToggle()), actionCollection(), "dvd_toggle");
 	new KAction(i18n("&Title"), NULL, 0, m_xine, SLOT(slotMenuTitle()), actionCollection(), "dvd_title");
@@ -1460,6 +1497,12 @@ void XinePart::loadConfig()
 	m_broadcastPort = config->readNumEntry("Port", 8080);
 	m_broadcastAddress = config->readEntry("Master Address", "localhost");
 
+	config->setGroup( "Video Settings" );
+	slotSetHue(config->readNumEntry( "Hue", -1));
+	slotSetSaturation(config->readNumEntry( "Saturation", -1));
+	slotSetContrast(config->readNumEntry( "Contrast", -1));
+	slotSetBrightness(config->readNumEntry( "Brigthness", -1));
+
 	m_equalizer->ReadValues(config);
 }
 
@@ -1489,6 +1532,12 @@ void XinePart::saveConfig()
 	config->writeEntry("Port", m_broadcastPort);
 	config->writeEntry("Master Address", m_broadcastAddress);
 
+	config->setGroup( "Video Settings" );
+	config->writeEntry( "Hue", m_hue );
+	config->writeEntry( "Saturation", m_saturation );
+	config->writeEntry( "Contrast", m_contrast );
+	config->writeEntry( "Brigthness", m_brightness );
+
 	m_equalizer->SaveValues(config);
 }
 
@@ -1514,6 +1563,7 @@ bool XinePart::isPaused()
 	return (m_xine->getSpeed() == KXineWidget::Pause);
 }
 
+#if 0
 void XinePart::audiocdMRLS(MRL::List& mrls, bool& ok, bool& supported, const QString& device)
 {
 	if (!m_xine->isXineReady())
@@ -1664,6 +1714,7 @@ void XinePart::dvdMRLS(MRL::List& mrls, bool& ok, bool& supported, const QString
 	if (mrls.count())
 		ok = true;
 }
+#endif
 
 bool XinePart::hasChapters()
 {
@@ -1999,9 +2050,4 @@ void VolumeSlider::wheelEvent(QWheelEvent* e)
 	}
 }*/
 
-
-/********/
-
-
-
 #include "xine_part.moc"
