@@ -65,9 +65,6 @@ public:
 	 */
 	bool isPlaying();
 	bool isPaused();
-	void audiocdMRLS(MRL::List& mrls, bool& ok, bool& supported, const QString& device);
-	void vcdMRLS(MRL::List& mrls, bool& ok, bool& supported, const QString& device);
-	void dvdMRLS(MRL::List& mrls, bool& ok, bool& supported, const QString& device);
 
 	bool hasChapters(); /* e.g. DVD */
 	void playNextChapter();
@@ -134,7 +131,7 @@ public slots:
 	void slotJumpIncrement(int);
 	void slotDelaySubTitle();
 	void slotAdvanceSubTitle();
-
+	void slotAddSubtitle();
 	/***************** Private ********************/
 
 private slots:
@@ -198,9 +195,6 @@ private:
 	void saveConfig();
 
 private:
-	QString m_hrefParam;
-	bool m_autoStart;
-	bool m_imageWindow;
 	QPoint m_oldPosition;
 	QTimer m_posCheckTimer;
 	QTimer m_osdTimerEnabler;  /* Provide Long click on timer button */
@@ -209,7 +203,7 @@ private:
 	int m_brightness, m_hue, m_contrast, m_saturation;
 
 	MRL m_mrl;
-	MRL::List m_playlist;
+	QValueList<MRL> m_playlist;
 	uint m_current;
 	uint m_lastDeinterlaceQuality;
 	QString m_lastDeinterlacerConfig;
